<?php


class WC_OS_Shipping
{

    private $selection;
    private $settings;
    private $order_total_default;
    private $order_total_criteria;
    private $original_order;

    /**
     * WC_OS_Shipping constructor.
     */
    public function __construct()
    {



        $this->settings = get_option('wc_os_shipping_settings', array());
        add_filter('wc_os_translation_array', array($this, 'wc_os_translation_array_filter'));
        add_action('wp_ajax_wc_os_save_shipping_settings', array($this, 'wc_os_save_shipping_settings_callback'));
        add_action('wc_os_after_order_split', array($this, 'wc_os_shipping_management_after_split'), 9, 2);
        add_action('wc_os_before_order_split', array($this, 'wc_os_shipping_management_before_split'));


    }

    public function is_parent_modified(){

        global $wc_os_effect_parent, $wc_os_settings;

        $method = $wc_os_settings['wc_os_ie'];

        $array_modified = array(
                'io', 'inclusive', 'exclusive', 'groups'
        );

        return !wc_os_order_removal() && $wc_os_effect_parent && in_array($method, $array_modified);

    }

    /**
     * @return mixed
     */
    public function get_order_total_default()
    {
        $this->order_total_default = array_key_exists('wc_os_order_total_default', $this->settings) ? $this->settings['wc_os_order_total_default']: 0;

        return $this->order_total_default;
    }



    /**
     * @return mixed
     */
    public function get_order_total_criteria()
    {
        $this->order_total_criteria = array_key_exists('wc_os_order_total_criteria', $this->settings) ? $this->settings['wc_os_order_total_criteria']: array();

        return $this->order_total_criteria;
    }



    /**
     * @return mixed
     */
    public function get_selection()
    {

        $this->selection = array_key_exists('wc_os_shipping_selection', $this->settings) ? $this->settings['wc_os_shipping_selection']: 'no';

        return $this->selection;
    }

    /**
     * @return mixed
     */
    public function get_settings()
    {
        return $this->settings;
    }

    public function wc_os_shipping_management_before_split($original_order){

        $original_order = new WC_Order($original_order);
        $this->original_order = $original_order;

    }


    public function wc_os_shipping_management_after_split($child_order_ids, $original_order_id){

//        $this->original_order->get

        switch ($this->get_selection()){



            case 'equal':


                    $this->wc_os_equal_shipping($child_order_ids, $original_order_id);

                break;

            case 'shipping_class':


                $this->wc_os_shipping_class_shipping($child_order_ids, $original_order_id);

                break;

            case 'clone':


                $this->wc_os_clone_shipping($child_order_ids, $original_order_id);

                break;

            case 'order_total':


                $this->wc_os_order_total_shipping($child_order_ids, $original_order_id);

                break;


            default:



                break;

        }


    }








    public function wc_os_save_shipping_settings_callback(){

        $result = array(
            'status' => false,
        );

        if(isset($_POST['wc_os_shipping_settings'])){

            if(!isset($_POST['wc_os_nonce']) || !wp_verify_nonce($_POST['wc_os_nonce'], 'wc_os_shipping_nonce_action')){

                wp_die(__('Sorry, your nonce did not verify.', 'woo-order-splitter'));

            }else{



                $wc_os_shipping_settings = sanitize_wc_os_data($_POST['wc_os_shipping_settings']);

                $result['status'] = update_option('wc_os_shipping_settings', $wc_os_shipping_settings);

            }


        }

        wp_send_json($result);


    }

    public function wc_os_translation_array_filter($translation_array){


        $shipping_translation = array(

            'shipping_nonce' => wp_create_nonce('wc_os_shipping_nonce_action'),
            'shipping_selection' => $this->get_selection(),
            'shipping_settings' => $this->settings,
            'negative_number_msg' => __('All values should be greater than or equal to zero.', 'woo-order-splitter')

        );

        $translation_array = array_merge($translation_array, $shipping_translation);

        return $translation_array;
    }

    public function wc_os_shipping_tab_html(){

        global $wc_os_url;

        ?>

        <div class="nav-tab-content hides wc_os_shipping">






                <h3 class="nav-tab-wrapper" >

                    <a class="nav-tab nav-tab-active" data-selection="no">
                        <?php _e("No Shipping",'woo-order-splitter'); ?>
                    </a>

                    <a class="nav-tab" data-selection="equal">
                        <?php _e("Equal Shipping",'woo-order-splitter'); ?>
                    </a>

                    <a class="nav-tab" data-selection="shipping_class">
                        <?php _e("Shipping Class Based",'woo-order-splitter'); ?>
                    </a>

                    <a class="nav-tab" data-selection="clone">
                        <?php _e("Clone Shipping",'woo-order-splitter'); ?>
                    </a>

                    <a class="nav-tab" data-selection="order_total">
                        <?php _e("Order Total Based",'woo-order-splitter'); ?>
                    </a>

                </h3>


                <div class="wc_os_alert alert-success hides">
                    <?php _e("Settings saved successfully.",'woo-order-splitter'); ?>
                </div>


                <div class="sub-tab-content no_shipping">

                    <div class="wc_os_alert">
                        <?php _e("Shipping cost will not be included in child orders if this this option is selected.",'woo-order-splitter'); ?>
                    </div>

                    <div>
                        <input type="radio" name="wc_os_shipping_selection" class="wc_os_shipping_selection" value="no" id="wc_os_no_shipping">
                        <label for="wc_os_no_shipping"><?php _e("No shipping cost for child orders",'woo-order-splitter'); ?></label>
                    </div>





                </div>

                <div class="sub-tab-content equal_shipping hides">

                    <div class="wc_os_alert">
                        <?php _e("Shipping cost will be divided equally among child orders.",'woo-order-splitter'); ?>
                    </div>

                    <div>
                        <input type="radio" name="wc_os_shipping_selection" class="wc_os_shipping_selection" value="equal" id="wc_os_equal_shipping">
                        <label for="wc_os_equal_shipping"><?php _e("Divide parent order shipping cost among child orders equally",'woo-order-splitter'); ?></label>
                    </div>

                </div>

                <div class="sub-tab-content shipping_class hides">

                    <div class="wc_os_alert">
                        <?php _e("Shipping cost will be added as per selected shipping class in individual product.",'woo-order-splitter'); ?>
                    </div>

                    <div>
                        <input type="radio" name="wc_os_shipping_selection" class="wc_os_shipping_selection" value="shipping_class"  id="wc_os_shipping_class">
                        <label for="wc_os_shipping_class"><?php _e("Add shipping cost as per shipping class selected in Product settings",'woo-order-splitter'); ?></label>
                    </div>

                </div>

                <div class="sub-tab-content clone_shipping hides">

                    <div class="wc_os_alert">
                        <?php _e("Shipping cost will be cloned from parent order.",'woo-order-splitter'); ?>
                    </div>

                    <div>
                        <input type="radio" name="wc_os_shipping_selection" class="wc_os_shipping_selection" value="clone" id="wc_os_clone_shipping">
                        <label for="wc_os_clone_shipping"><?php _e("Clone shipping cost from parent order",'woo-order-splitter'); ?></label>
                    </div>


                </div>

                <div class="sub-tab-content order_total hides">

                    <div class="wc_os_alert">
                        <?php _e("Shipping cost will be added as per defined criteria according to the order total.",'woo-order-splitter'); ?>
                    </div>

                    <div>
                        <input type="radio" name="wc_os_shipping_selection" class="wc_os_shipping_selection" value="order_total" id="wc_os_order_total_shipping">
                        <label for="wc_os_order_total_shipping"><?php _e("Apply shipping cost as per each order total",'woo-order-splitter'); ?></label>
                    </div>


                    <div>
                        <label for="wc_os_order_total_default" class="wc_os_label"><?php _e("Default Value (If no criteria matched)",'woo-order-splitter'); ?></label>
                        <input type="number" min="0" id="wc_os_order_total_default" value="0">
                    </div>

                    <div>
                        <h3><?php _e("Define Criteria",'woo-order-splitter'); ?></h3>
                    </div>


                    <div class="wc_os_order_total_criteria_wrapper hides">
                        <span>
                           <?php _e("Min total",'woo-order-splitter'); ?>
                        </span>

                        <span>
                           <?php _e("Max total",'woo-order-splitter'); ?>
                        </span>

                        <span>
                           <?php _e("Shipping Cost",'woo-order-splitter'); ?>
                        </span>

                    </div>

                    <div class="wc_os_order_total_criteria_row_sample hides" >

                        <input type="number" min="0" value="" name="min">
                        <input type="number" min="0" value="" name="max">
                        <input type="number" min="0" value="" name="cost">
                        <button class="button button-secondary wc_os_del_total_order_criteria" title="<?php _e("Delete row",'woo-order-splitter'); ?>"><span class="dashicons dashicons-trash"></span></button>


                    </div>

                    <div class="wc_os_add_total_order_criteria">
                        <button class="button button-secondary" title="<?php _e("Add row",'woo-order-splitter'); ?>"><span class="dashicons dashicons-plus"></span> <?php _e("Add",'woo-order-splitter'); ?></button>
                    </div>




                </div>



                <button class="button button-primary save_changes_btn">
                    <?php _e("Save Changes",'woo-order-splitter'); ?>
                </button>


                <div id="wc_os_load_modal" class="wc_os_modal">

                    <!-- Modal content -->
                    <div class="wc_os_modal_content">
                        <img src="<?php echo $wc_os_url.'/img/juggler.gif' ?>" alt="" style="width: 50px; height: auto" />
                    </div>

                </div>





        </div>




        <?php

    }

    public function wc_get_min_max_cost($get_default_criteria){

        $min_max_array = array();
        if(!empty($get_default_criteria)){

            $min_index = -1;
            $max_index = -1;
            $min_val = -1;
            $max_val = -1;

            foreach ($get_default_criteria as $index => $single_criteria){

                $min = $single_criteria['min'];
                $max = $single_criteria['max'];
                $cost = $single_criteria['cost'];

                if($min_val == -1){
                    $min_val = $min;
                    $min_index = $index;


                }else{

                    if($min < $min_val){

                        $min_val = $min;
                        $min_index = $index;
                    }


                }

                if($max_val == -1){

                    $max_val = $max;
                    $max_index = $index;



                }else{


                    if($max > $max_val){

                        $max_val = $max;
                        $max_index = $index;

                    }

                }



            }

            $min_max_array = array(

                    'min' => $min_index,
                    'max' => $max_index,
            );

        }


        return $min_max_array;



    }

    public function wc_os_get_shipping_by_order_total($order_total){

        $default_shipping = $this->get_order_total_default();
        $get_default_criteria = $this->get_order_total_criteria();


        usort($get_default_criteria, function($a, $b){

            return $a['min'] > $b['min'];

        });

        $cost_actual = null;


        if(!empty($get_default_criteria)){

            foreach ($get_default_criteria as $index => $single_criteria){

                $min = $single_criteria['min'];
                $max = $single_criteria['max'];
                $cost = $single_criteria['cost'];

                if(!$min){


                    if($order_total <= $max){

                        $cost_actual = $cost;
                        break;

                    }

                }else{

                    if(!$max){

                        if($order_total >= $min){

                            $cost_actual = $cost;
                            break;

                        }

                    }else{

                        if($order_total > $min && $order_total < $max){

                            $cost_actual = $cost;
                            break;
                        }

                    }

                }

            }





        }else{
            $cost_actual = $default_shipping;

        }

        if(is_null($cost_actual)){


            $cost_actual = $default_shipping;


            $min_max = $this->wc_get_min_max_cost($get_default_criteria);

            if(!empty($min_max)){

                $min_val = $get_default_criteria[$min_max['min']]['min'];
                $min_cost = $get_default_criteria[$min_max['min']]['cost'];
                $max_val = $get_default_criteria[$min_max['max']]['max'];
                $max_cost = $get_default_criteria[$min_max['max']]['cost'];

                if($order_total < $min_val){
                    $cost_actual = $min_cost;
                }elseif($order_total > $max_val){

                    $cost_actual = $max_cost;

                }

            }


        }



        return $cost_actual;

    }

    public function wc_os_save_shipping_item_against_order(WC_Order_Item_Shipping $shipping_item){

        global $wpdb;
        $order_item_table = $wpdb->prefix.'woocommerce_order_items';
        $order_item_meta_table = $wpdb->prefix.'woocommerce_order_itemmeta';
        $order_id = $shipping_item->get_order_id();
        $order_item_type = 'shipping';
        $order_item_name = $shipping_item->get_name();

        $method_id = $shipping_item->get_method_id();
        $instance_id = $shipping_item->get_instance_id();
        $cost = $shipping_item->get_total();
        $total_tax = $shipping_item->get_total_tax();
        $taxes = maybe_serialize($shipping_item->get_taxes());

        $insert_query = "INSERT INTO $order_item_table (order_item_name, order_item_type, order_id) VALUES ('$order_item_name', '$order_item_type', $order_id)";

        $result = $wpdb->query($insert_query);

        if($result){

            $item_id_query = "SELECT order_item_id FROM $order_item_table WHERE order_id = $order_id AND order_item_type = '$order_item_type'";

            $order_item_id = $wpdb->get_var($item_id_query);

            $insert_query_meta = "INSERT INTO $order_item_meta_table (order_item_id, meta_key, meta_value) 
                            VALUES
                            
                             ($order_item_id, 'method_id', '$method_id'),
                             ($order_item_id, 'instance_id', '$instance_id'),
                             ($order_item_id, 'cost', '$cost'),
                             ($order_item_id, 'total_tax', $total_tax),
                             ($order_item_id, 'taxes', '$taxes')
                             
                             
                             ";

            $wpdb->query($insert_query_meta);


            $item_id_query = "SELECT * FROM $order_item_meta_table WHERE order_item_id = $order_item_id";

            $order_item_id = $wpdb->get_results($item_id_query);

            //wc_os_pree($order_item_id);



        }





    }

    public function wc_os_get_shipping_item($parent_shipping_data){


        $new_shipping_item = new WC_Order_Item_Shipping();

        $new_shipping_item->set_instance_id($parent_shipping_data['instance_id']);
        $new_shipping_item->set_method_id($parent_shipping_data['method_id']);
        $new_shipping_item->set_method_title('WC OS Custom Shipping');
        $new_shipping_item->set_name('WC OS Custom Shipping');
//        $new_shipping_item->set_method_title($parent_shipping_data['method_title']);
//        $new_shipping_item->set_name($parent_shipping_data['name']);

        return $new_shipping_item;

    }

    private function wc_os_equal_shipping($child_order_ids, $original_order_id)
    {


        $parent_order = $this->original_order;
		$parent_order = (!empty($parent_order)?$parent_order:wc_get_order($original_order_id));
		
        $parent_shipping = $parent_order->get_items('shipping');
        $parent_shipping_id = current(array_keys($parent_shipping));
        if(!$parent_shipping){return;}

        $parent_shipping = current($parent_shipping);
        $parent_shipping_data = $parent_shipping->get_data();
        $shipping_total = $parent_shipping_data['total'];

        if(empty($child_order_ids)){return;}

        if($this->is_parent_modified()){
            wc_delete_order_item($parent_shipping_id);
            $child_order_ids[] = $original_order_id;
        }

        $total_child = count($child_order_ids);
        $each_child_shipping = $shipping_total/$total_child;
        $each_child_shipping = round($each_child_shipping, 2);


        if(!empty($child_order_ids)){
            foreach ($child_order_ids as $child_order_id){

                $child_order = new WC_Order($child_order_id);
				$this->wc_os_shipping_remove_shipping_items($child_order);


                $shipping_item = $this->wc_os_get_shipping_item($parent_shipping_data);

                $shipping_item->set_method_title($parent_shipping_data['method_title']);
                $shipping_item->set_name($parent_shipping_data['name']);
                $shipping_item->set_total($each_child_shipping);
                $shipping_item->set_order_id($child_order_id);
                $child_order->add_item($shipping_item);
                $child_order->calculate_totals();
                $child_order->save();


            }


        }



    }

    private function wc_os_order_total_shipping($child_order_ids, $original_order_id)
    {

//        delete_post_meta($original_order_id, 'split_status');

        $parent_order = $this->original_order;
		$parent_order = (!empty($parent_order)?$parent_order:wc_get_order($original_order_id));
		
        $parent_shipping = $parent_order->get_items('shipping');
        $parent_shipping_id = current(array_keys($parent_shipping));
        if(!$parent_shipping){return;}
        $parent_shipping = current($parent_shipping);
        $parent_shipping_data = $parent_shipping->get_data();

        if($this->is_parent_modified()){
            wc_delete_order_item($parent_shipping_id);
            $child_order_ids[] = $original_order_id;
        }

        if(!empty($child_order_ids)){
            foreach ($child_order_ids as $child_order_id){

                $child_order = new WC_Order($child_order_id);
				$this->wc_os_shipping_remove_shipping_items($child_order);
				
                $child_order_total = $child_order->get_total();

                $shipping_total = $this->wc_os_get_shipping_by_order_total($child_order_total);

                $shipping_item = $this->wc_os_get_shipping_item($parent_shipping_data);

                $shipping_item->set_total($shipping_total);
                $shipping_item->set_order_id($child_order_id);
				
				

//                $this->wc_os_save_shipping_item_against_order($shipping_item);

                $child_order->add_item($shipping_item);
                $child_order->calculate_totals();
                $child_order->save();


            }


        }



    }

    private function wc_os_clone_shipping($child_order_ids, $original_order_id)
    {


        $parent_order = $this->original_order;
		$parent_order = (!empty($parent_order)?$parent_order:wc_get_order($original_order_id));
        $parent_shipping = $parent_order->get_items('shipping');
        $parent_shipping_id = current(array_keys($parent_shipping));
        if(!$parent_shipping){return;}

        $parent_shipping = current($parent_shipping);
        $parent_shipping_data = $parent_shipping->get_data();
        $shipping_total = $parent_shipping_data['total'];

        if(empty($child_order_ids)){return;}

        if($this->is_parent_modified()){
            wc_delete_order_item($parent_shipping_id);
            $child_order_ids[] = $original_order_id;
        }

        if(!empty($child_order_ids)){

            foreach ($child_order_ids as $child_order_id){

                $child_order = new WC_Order($child_order_id);
				$this->wc_os_shipping_remove_shipping_items($child_order);


                $shipping_item = $this->wc_os_get_shipping_item($parent_shipping_data);

                $shipping_item->set_method_title($parent_shipping_data['method_title']);
                $shipping_item->set_name($parent_shipping_data['name']);
                $shipping_item->set_total($shipping_total);
                $shipping_item->set_order_id($child_order_id);
                $child_order->add_item($shipping_item);
                $child_order->calculate_totals();
                $child_order->save();


            }


        }



    }
	
	private function wc_os_shipping_remove_shipping_items($order){
	
		$child_shipping = $order->get_items('shipping');
		
		if(!empty($child_shipping)){			
			$child_shipping_ids = array_keys($child_shipping);			
			foreach($child_shipping_ids as $child_shipping_id){			
				//wc_delete_order_item($child_shipping_id);		
				$order->remove_item( $child_shipping_id );	
			}
		}
		
		$order->calculate_shipping();
	
	}
	private function wc_os_shipping_class_shipping($child_order_ids, $original_order_id) {
	
		//		 delete_post_meta($original_order_id, 'split_status');
	
		$packages = array();
		$parent_order = $this->original_order;
		$parent_order = (!empty($parent_order)?$parent_order:wc_get_order($original_order_id));
	
		if (empty($parent_order)) {
			return;
		}
	
		$parent_shipping_items = $parent_order->get_items('shipping');
	
	
		if (empty($parent_shipping_items)) {
			return;
		}
	
		if (empty($child_order_ids)) {
			return;
		}
	
	
		$method_ids_selected_parent = array();
		$parent_shipping_data = array();
		$parent_rate_ids = array();
	
		foreach($parent_shipping_items as $shiping_item_id => $parent_shipping) {
	
	
			$parent_method_id = $parent_shipping->get_method_id();
			$parent_instance = $parent_shipping->get_instance_id();
			$rate_id = $parent_method_id.
			':'.$parent_instance;
			$parent_rate_ids[] = $rate_id;
	
	
			$method_ids_selected_parent[] = $parent_method_id;
	
	
			$vendor_id = $parent_shipping->get_meta('vendor_id');
			$method_slug = $parent_shipping->get_meta('method_slug');
			if ($vendor_id) {
	
				$parent_shipping_data[$vendor_id]['shipping'] = $parent_shipping;
				$parent_shipping_data[$vendor_id]['method_slug'] = $method_slug;
	
			} else {
	
				$parent_shipping_data[0]['shipping'] = $parent_shipping;
				$parent_shipping_data[0]['method_slug'] = $method_slug;
	
			}
	
		}
	
	
		if ($this->is_parent_modified()) {
	
			$parent_shipping_ids = array_keys($parent_shipping_items);
	
			foreach($parent_shipping_ids as $parent_shipping_id) {
	
				//wc_delete_order_item($parent_shipping_id);
				$parent_order->remove_item($parent_shipping_id);
				$parent_order->calculate_shipping();
	
			}
	
			$child_order_ids[] = $original_order_id;
		}
	
		if (!empty($child_order_ids)) {
	
			foreach($child_order_ids as $child_order_id) {
	
				$child_order = new WC_Order($child_order_id);
				$child_vendor_ids = get_post_meta($child_order_id, '_group_vendor_ids', true);
	
				$this->wc_os_shipping_remove_shipping_items($child_order);
				
				if(!is_admin()){
		
					if (is_object(WC()->cart) && !WC()->cart->is_empty() ) {
						WC()->cart->empty_cart();
					}else{
						global $woocommerce;
						if(is_object($woocommerce)){
							$woocommerce->cart->empty_cart();
						}
					}
					
				}
	
				foreach($child_order->get_items() as $order_Item) {
	
					$order_Item_Data = $order_Item->get_data();
	
					$order_Item_id = $order_Item_Data['product_id'];
					$order_Item_qty = $order_Item->get_quantity();
					
					if(!is_admin()){
						WC()->cart->add_to_cart($order_Item_id, $order_Item_qty);
					}
	
				}
				
				if(!is_admin()){	
					$packages = WC()->shipping()->get_packages();
				}
	
				if (empty($packages)) {
					continue;
				}
	
				$current_package = current($packages);
				$rates = $current_package['rates'];
	
	
	
				if (!empty($rates)) {
	
					foreach($rates as $rate_key => $rate_object) {
	
	
						$shipping_included = array();
						if (!empty($child_vendor_ids)) {
	
							foreach($child_vendor_ids as $child_vendor_id) {
	
	
	
								if (array_key_exists($child_vendor_id, $parent_shipping_data)) {
	
									$parent_shipping_c = $parent_shipping_data[$child_vendor_id];
	
	
								} else {
	
									$parent_shipping_c = array_key_exists(0, $parent_shipping_data) ? $parent_shipping_data[0] : false;
	
								}
	
	
								if ($parent_shipping_c) {
	
									$parent_shipping_item = $parent_shipping_c['shipping'];
									$method_slug_vendor = $parent_shipping_c['method_slug'];
									$rate_object_id = $rate_object->get_id();
									$rate_object_slug = explode(':', $rate_object_id);
									$rate_object_slug = current($rate_object_slug);
									// $meta_data = $parent_shipping_item->get_meta_data();
	
									if ($rate_object_slug == $method_slug_vendor) {
	
										$shipping_item = new WC_Order_Item_Shipping();
	
										$shipping_item->set_method_title($parent_shipping_item->get_method_title());
										$shipping_item->set_method_id($parent_shipping_item->get_method_id());
										$shipping_item->set_instance_id($parent_shipping_item->get_instance_id());
										$shipping_item->set_total($parent_shipping_item->get_total());
										$shipping_item->set_taxes($parent_shipping_item->get_taxes());
	
										$parent_meta = $parent_shipping_item->get_meta_data();
	
										if (!empty($parent_meta)) {
	
											foreach($parent_meta as $meta_data) {
	
	
												$data_array = $meta_data->get_data();
	
												$shipping_item->add_meta_data($data_array['key'], $data_array['value']);
	
											}
	
										}
	
										$shipping_item->save_meta_data();
										$shipping_item->set_order_id($child_order_id);
										$child_order->add_item($shipping_item);
										$shipping_included[] = true;
	
	
									}
	
								}
	
							}
	
						}
	
						if (empty($child_vendor_ids) || empty($shipping_included)) {
	
							if (in_array($rate_key, $parent_rate_ids)) {
	
								$shipping_item = new WC_Order_Item_Shipping();
	
								$shipping_item->set_shipping_rate($rate_object);
								$shipping_item->set_order_id($child_order_id);
								$child_order->add_item($shipping_item);
	
	
							}
	
						}
	
					}
	
				}
	
				$child_order->calculate_shipping();
				$child_order->calculate_totals();
				$child_order->save();
	
				WC()->cart->empty_cart();
	
	
	
			}
	
	
		}
	
	
	
	}
}